"""
Copyright CNRS/Inria/UniCA
Contributor(s): Eric Debreuve (eric.debreuve@cnrs.fr) since 2018
SEE COPYRIGHT NOTICE BELOW
"""

import typing as h
from pathlib import Path as path_t

import numpy as nmpy
from obj_mpp.interface.storage.api import signal_details_h

array_t = nmpy.ndarray


def SignalByNumpy(
    signal_path: path_t | None, /, *, can_be_anything: bool = True
) -> signal_details_h:
    """"""
    if signal_path is None:
        return None, None

    return _SignalByNumpy(signal_path, can_be_anything=can_be_anything)


def _SignalByNumpy(
    path: path_t, /, *, can_be_anything: bool = True
) -> tuple[array_t | None, Exception | None]:
    """"""
    try:
        signal = nmpy.load(str(path))
        error = None
    except Exception as exception:
        signal = None
        error = exception

    if can_be_anything:
        return signal, error

    if isinstance(signal, h.Mapping):
        keys = tuple(signal.keys())
        signal = signal[keys[0]]
    if isinstance(signal, array_t):
        return signal, None

    return None, ValueError(
        f'Invalid signal loaded for "{path}": '
        f"Actual={signal}; "
        f'Expected=a Numpy array of type "Numpy.ndarray".'
    )


"""
COPYRIGHT NOTICE

This software is governed by the CeCILL  license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

SEE LICENCE NOTICE: file README-LICENCE-utf8.txt at project source root.

This software is being developed by Eric Debreuve, a CNRS employee and
member of team Morpheme.
Team Morpheme is a joint team between Inria, CNRS, and UniCA.
It is hosted by the Centre Inria d'Université Côte d'Azur, Laboratory
I3S, and Laboratory iBV.

CNRS: https://www.cnrs.fr/index.php/en
Inria: https://www.inria.fr/en/
UniCA: https://univ-cotedazur.eu/
Centre Inria d'Université Côte d'Azur: https://www.inria.fr/en/centre/sophia/
I3S: https://www.i3s.unice.fr/en/
iBV: http://ibv.unice.fr/
Team Morpheme: https://team.inria.fr/morpheme/
"""
